/*
 * USB Nomad Jukebox filesystem for Linux.
 *
 * Copyright (C) 2001 Jrg Prante <joergprante@gmx.de>
 *
 * based on ftpfs by Florin Malita <fmalita@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/smp_lock.h>
#include <linux/fs.h>
#include <linux/usb.h>
#include <linux/proc_fs.h>	/* /proc filesystem */
#include <asm/system.h>
#include <asm/uaccess.h>

#include "njbfs.h"
#include "njb_usb.h"
#include "njbfs_proc.h"
#include "nomad_usb.h"

#if defined(CONFIG_PROC_FS)

#define CHECK(x) ((x) ? "Yes" : "No")
#define CHOOSE(x,y,a,b) ((x == y) ? a : b)

/* njb_usb */
extern int njb_usb_ping(struct nomad_usb_data *nomad, njbid_t * njbid);

static struct proc_dir_entry *proc_entry = NULL;
static struct proc_dir_entry *proc_dir = NULL;
static struct proc_dir_entry *proc_driver = NULL;


/* /proc/njb/driver */

static int njb_read_proc_driver(char *page, char **start, off_t off,
				int count, int *eof, void *data)
{
	char *out = page;
	int len;
	struct nomad_usb_data *nomad = (struct nomad_usb_data *) data;
	njbid_t njbid;
	int status;
	int i;


	out += sprintf(out, "Nomad Jukebox Driver Status\n");
	out += sprintf(out, "---------------------------\n");
	out +=
	    sprintf(out, "NJB File System Version : %s\n", NJBFS_VERSION);

	if (!nomad) {
		err("no data");
		goto out;
	}

	out += sprintf(out, "Connected : %s\n", CHECK(nomad->present));

	if (nomad->present) {

		/* ping device */
		status = njb_usb_ping(nomad, &njbid);
		if (status == 0) {
			out += sprintf(out, "Jukebox ID     : ");
			for (i = 0; i < 16; i++) {
				out += sprintf(out, "%02x ", njbid.id[i]);
			}
			out += sprintf(out, "\nFirmware       : %d.%d\n",
				       njbid.fwMajor, njbid.fwMinor);
			out +=
			    sprintf(out, "Product Name   : %s\n",
				    njbid.productName);
			switch (njbid.power) {
			case NJB_POWER_BATTERY:
				out +=
				    sprintf(out,
					    "Power          : battery\n");
				break;
			case NJB_POWER_AC_CHARGED:
				out +=
				    sprintf(out,
					    "Power          : AC charged\n");
				break;
			case NJB_POWER_AC_CHARGING:
				out +=
				    sprintf(out,
					    "Power          : AC charging\n");
				break;
			default:
				out +=
				    sprintf(out, "Power          : ?\n");
				break;
			}
		} else {
			out +=
			    sprintf(out, "Can't ping, status = %d\n",
				    status);
		}
	}

      out:

	len = out - page;
	len -= off;
	if (len < count) {
		*eof = 1;
		if (len <= 0)
			return 0;
	} else
		len = count;
	*start = page + off;
	return len;
}

/**
 *  Create a /proc/njb entry
 *  The entry /proc/njb/driver provides some information about the
 *  driver
 */
static int proc_njb_create(struct nomad_usb_data *nomad)
{
	if (!nomad)
		return -1;

	proc_dir = create_proc_entry("njb", S_IFDIR, proc_entry);
	if (proc_dir == NULL) {
		err("proc_njb_create: could not create/proc/njb entry");
		return -1;
	}
	info("/proc/njb created");

	if (proc_entry != NULL)
		proc_entry->owner = THIS_MODULE;

	proc_driver = create_proc_read_entry("driver",
					     S_IFREG | S_IRUGO | S_IWUSR,
					     proc_dir,
					     njb_read_proc_driver, nomad);
	if (!proc_driver) {
		err
		    ("proc_njb_create: could not create /proc/njb/driver entry");
		return -1;
	}
	info("/proc/njb/driver created");

	return 0;
}

static void proc_njb_destroy()
{
	if (!proc_driver)
		return;

	remove_proc_entry("driver", proc_dir);
	proc_driver = NULL;

	if (!proc_dir) {
		err("proc_njb_destroy: could not destroy /proc/njb");
		return;
	}

	remove_proc_entry("njb", proc_entry);
	proc_dir = NULL;

	return;
}

EXPORT_SYMBOL(proc_njb_create);
EXPORT_SYMBOL(proc_njb_destroy);

#endif
