#ifndef _NJBFS_CACHE_H_
#define _NJBFS_CACHE_H_

#define NJBFS_CACHE_LEN	5
#define NJBFS_CACHE_HASH	5
#define NJBFS_CACHE_TTL 300

/* ----------------- NJB frames -------------------------- */

#define ID_DATA_ASCII	0
#define ID_DATA_BIN	1

#define FR_SIZE		"FILE SIZE"
#define FR_LENGTH	"LENGTH"
#define FR_CODEC	"CODEC"
#define FR_TITLE	"TITLE"
#define FR_ALBUM	"ALBUM"
#define FR_GENRE	"GENRE"
#define FR_ARTIST	"ARTIST"
#define FR_TRACK	"TRACK NUM"
#define FR_FNAME	"FNAME"
#define FR_YEAR		"YEAR"

#define NJB_CODEC_MP3	"MP3"
#define NJB_CODEC_WMA	"WMA"
#define NJB_CODEC_WAV	"WAV"


typedef struct {
	unsigned char id[16];
	u_int8_t fwMajor;
	u_int8_t fwMinor;
	char productName[33];
	u_int8_t power;
} njbid_t;

typedef struct {
	unsigned char id[16];
	u_int64_t count;
} njblibctr_t;

typedef unsigned char eax_t;

typedef struct playlist_track_struct {
	u_int32_t trackid;
	struct playlist_track_struct *prev;
	struct playlist_track_struct *next;
} playlist_track_t;

typedef struct playlist_struct {
	char *name;
	int _state;
#define NJB_PL_NEW		0
#define NJB_PL_UNCHANGED	1
#define NJB_PL_CHNAME		2
#define NJB_PL_CHTRACKS		3
	u_int32_t ntracks;
	u_int32_t plid;
	playlist_track_t *first;
	playlist_track_t *last;
	playlist_track_t *cur;
} playlist_t;

typedef struct {
	u_int32_t trackid;
	u_int32_t size;
} njbttaghdr_t;


typedef struct track_frame_struct {
	u_int16_t type;
	u_int16_t labelsz;
	u_int16_t datasz;
	char *label;
	void *data;
	struct track_frame_struct *next;
} track_frame_t;

typedef struct track_struct {
	njbttaghdr_t header;
	u_int16_t nframes;
	track_frame_t *first;
	track_frame_t *last;
	track_frame_t *cur;
} track_t;


#define OWNER_STRING_LENGTH	128
typedef unsigned char owner_string[OWNER_STRING_LENGTH];


typedef struct {
	u_int32_t plid;
	u_int32_t size;
} njbplhdr_t;

typedef struct {
	u_int32_t dfid;
	u_int32_t size;
} njbdfhdr_t;

typedef struct {
	char *filename;
	u_int32_t dfid;
	u_int32_t msdw;
	u_int32_t lsdw;
} datafile_t;


/* ----------------- NJB file system cache ------------------------ */


struct njbfs_dir_entry {
	umode_t mode;
	char *name;
	off_t size;
	nlink_t nlink;
	unsigned long blocksize;
	unsigned long blocks;
	time_t atime;
	u_int32_t fileid;	/* the NJB file id */
	/* NJB attributes */
	char *codec;		/* codec frame */
	char *title;		/* title frame */
	char *artist;		/* artist frame */
	char *album;		/* album frame */
	char *genre;		/* genre frame */
	u_int32_t length;	/* optional frame: length in seconds */
	u_int32_t tracknum;	/* optional frame: album track number */
	track_t *track;		/* track */
	int update_track_tag;	/* should track tag be updated? 1=yes 0=no */
};

struct njbfs_dirlist_node {
	struct njbfs_dirlist_node *prev, *next;
	struct njbfs_dir_entry entry;
};

struct njbfs_directory {
	struct njbfs_dirlist_node *head;
	int valid;
	time_t time;
	char *name;
};

struct njbfs_hashlist_node {
	struct njbfs_hashlist_node *prev, *next;
	struct njbfs_directory directory;
};

struct njbfs_dir_cache {
	struct njbfs_hashlist_node *hash[NJBFS_CACHE_HASH];
	unsigned len[NJBFS_CACHE_HASH];
};

struct njbfs_sb_info;

void njbfs_cache_init(struct njbfs_sb_info *);
int njbfs_cache_empty(struct njbfs_sb_info *);
int njbfs_cache_get(struct njbfs_sb_info *, char *,
		    struct njbfs_directory **);
void njbfs_cache_invalidate(struct dentry *);

#endif
