#!/usr/bin/perl -w
use strict;

#written: Tue Jan  1 14:47:18 EST 2002
#author: mental@neverlight.com
# 
#This program is just a quick proof of concept piece I was inspired to 
#write after seeing njbfs.
#
#I needed a tool to set missing id3 tags and copy lots of files to the 
#nomad jukebox. 
#
#This is a handy module I found on cpan, debian users can 
#apt-get install libmp3-info-perl 
#Other distributions may have it packaged, if not download it from 
#cpan.org.
use MP3::Info;

#These (should) be core perl modules. 
use File::Find;
use File::Copy;
use Getopt::Std;

our $opt_c;
our $opt_e;
our $opt_m;

getopts('cem');
my $dir = shift || die usage();
die usage() unless -d $dir;
finddepth( \&fix_missing_tags, $dir ) if $opt_m || $opt_e;
finddepth( \&copy_files,       $dir ) if $opt_c;

sub fix_missing_tags {
    my $file = $_;

    #I only want mp3's. At least until Creative decides to support .ogg's 
    if ( $file =~ /\.mp3$/ ) {
        my $taginfo = get_mp3tag($file);
        return undef if ($taginfo) && !defined($opt_e);
        my $fileinfo = get_mp3info($file);
        print "Editing data for $file\n";
        get_taginfo( \$taginfo, 'ARTIST' );
        get_taginfo( \$taginfo, 'TITLE' );
        print "Storing data....\n";
        # store tag info
        set_mp3tag( $file, $taginfo )
          || warn "Unable to save tag info for this file $!\n";
    }
}

sub get_taginfo {
    my $tagref       = shift;
    my $tag          = shift;
    my $confirmation = 'n';
    while ( 'y' ne $confirmation || 'Y' ne $confirmation ) {

        if ( ${$tagref}->{$tag} ) {
            print( "\nTag is set to:\"", ${$tagref}->{$tag}, "\"\n" );
            my $edit_flag = input_data("Edit tag? (y/n):");
            return undef unless ( 'y' eq $edit_flag || 'Y' eq $edit_flag );
        }

        ${$tagref}->{$tag} = input_data("\nEnter $tag information:");
        $confirmation =
          input_data( 'You entered: '
            . ${$tagref}->{$tag}
            . '. Is this correct (Y/N)?:' );
    }
}

sub input_data {
    my $string = shift;
    print "$string";
    my $input = <STDIN>;
    chomp($input);
    return $input;
}

sub copy_files {
    my $file = $_;

    #I only want mp3's. At least until Creative decides to support .ogg's 
    if ( $file =~ /\.mp3$/ ) {

        #if there's no id3tag, skip it.
        if ( my $taginfo = get_mp3tag($file) ) {
            my $fileinfo = get_mp3info($file);

            #Make sure all the required data is there.
            next unless ( $taginfo->{'TITLE'} && $taginfo->{'ARTIST'} );

            #Ok, this is kind of ugly, but it needs done.
            my $destfile =
              "/mnt/njbfs/tracks/"
              . $taginfo->{'ARTIST'} . " - "
              . $taginfo->{'TITLE'}
              . ".mp3,size="
              . $fileinfo->{'SIZE'};
            print("Copyiny file to:$destfile\n\n"); #I like feedback...
            copy ($file,$destfile)||die "Unable to copy file:$file\n";
        }
    }
}

sub usage {
    print "Usage: $0 <opts> <path>\n\n";
    print "Opts are:\n", "copy files to nomad jukebox: -c\n";
    print "override (edit) existing id3tags: -e\n";
    print "find and edit missing tags: -m\n\n";
    print "Options are additive. eg:\n";
    print "$0 -e -c ~/mp3\n";
    print "Will first let you edit all id3tags, then copy every mp3 that has an id3tag in ~/mp3 to the Nomad Jukebox.\n";
    exit;
}
